﻿using ST_LINK_MES.Model;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace ST_LINK_MES.Service
{
    public class STLinkCliPrograrmService
    {
        public STLinkCliPrograrmService(string stlCliPath = "", string port = "", string customDefaultOption = null)
        {
            this._cli = new STLinkCliWrapService(stlCliPath, port, customDefaultOption);
        }

        public delegate void OnMsgRecevicedEvent(string msg);
        public event OnMsgRecevicedEvent OnMsgReceviced;

        public delegate void OnProgressChangedEvent(int currentStep,int maxStep);
        public event OnProgressChangedEvent OnProgressChanged;
        public string CliPath => _cli.CliPath;

        private readonly STLinkCliWrapService _cli;

        public Result StartProgramProcess(string filePath)
        {
            Result result = new Result() { IsSuccess = false };

            result.Step = 1;
            var setOptionsResult = SetOptions();
            if (setOptionsResult != null)
            {
                result.ErrorMsg = setOptionsResult.ErrorMsg;
                foreach (var pair in setOptionsResult.Data)
                {
                    result.Data[pair.Key] = pair.Value;
                }
            }
            if (setOptionsResult == null || !setOptionsResult.IsSuccess)
            {
                return result;
            }
            OnProgressChanged?.Invoke(1, 5);

            result.Step = 2;
            var startProgramResult = StartProgram(filePath);
            if (startProgramResult != null)
            {
                result.ErrorMsg = startProgramResult.ErrorMsg;
                foreach (var pair in startProgramResult.Data)
                {
                    result.Data[pair.Key] = pair.Value;
                }
            }
            if (startProgramResult == null || !startProgramResult.IsSuccess)
            {
                return result;
            }
            OnProgressChanged?.Invoke(2, 5);

            result.Step = 3;
            var restResult = Rest();
            if (restResult != null)
            {
                result.ErrorMsg = restResult.ErrorMsg;
                foreach (var pair in restResult.Data)
                {
                    result.Data[pair.Key] = pair.Value;
                }
            }
            if (restResult == null || !restResult.IsSuccess)
            {
                return result;
            }
            OnProgressChanged?.Invoke(3, 5);

            result.Step = 4;
            var checkResult = Check(filePath);
            if (checkResult != null)
            {
                result.ErrorMsg = checkResult.ErrorMsg;
                foreach (var pair in checkResult.Data)
                {
                    result.Data[pair.Key] = pair.Value;
                }
            }
            if (checkResult == null || !checkResult.IsSuccess)
            {
                return result;
            }
            OnProgressChanged?.Invoke(4, 5);

            result.Step = 5;
            var getCheckSumResult = GetCheckSum(filePath);
            if (getCheckSumResult != null)
            {
                result.ErrorMsg = getCheckSumResult.ErrorMsg;
                foreach (var pair in getCheckSumResult.Data)
                {
                    result.Data[pair.Key] = pair.Value;
                }
            }
            if (getCheckSumResult == null || !getCheckSumResult.IsSuccess)
            {
                return result;
            }
            OnProgressChanged?.Invoke(5, 5);

            result.IsSuccess = true;
            return result;
        }

        public Result SetOptions()
        {
            var result = new Result() { IsSuccess = true };

            _cli.OnMsgReceviced += stlCli_OnMsgReceviced_SetDefaultOptions;
            var resultInt = _cli.SetDefaultOptions();
            _cli.OnMsgReceviced -= stlCli_OnMsgReceviced_SetDefaultOptions;

            result.IsSuccess = resultInt == 0;
            return result;

            void stlCli_OnMsgReceviced_SetDefaultOptions(string msg)
            {
                if (msg is null)
                    return;

                if (msg.Contains("Unable to connect to ST-LINK!"))
                {
                    result.ErrorMsg = "ERROR: Unable to connect to ST-LINK!";
                }
                OnMsgReceviced?.Invoke(msg);
            }
        }

        public Result StartProgram(string filePath)
        {
            if (!File.Exists(filePath))
            {
                return new Result()
                {
                    IsSuccess = false,
                    ErrorMsg = "ERROR: Program file not found",
                };
            }

            var result = new Result() { IsSuccess = true };

            _cli.OnMsgReceviced += stlCli_OnMsgReceviced_StartProgram;
            var resultInt = _cli.StartProgram(filePath);
            _cli.OnMsgReceviced -= stlCli_OnMsgReceviced_StartProgram;

            result.IsSuccess = resultInt == 0;
            return result;

            void stlCli_OnMsgReceviced_StartProgram(string msg)
            {
                if (msg is null)
                    return;

                if (msg.Contains("Unable to connect to ST-LINK!"))
                {
                    result.ErrorMsg = "ERROR: Unable to connect to ST-LINK!";
                }
                OnMsgReceviced?.Invoke(msg);
            }
        }

        public Result Rest()
        {
            var result = new Result() { IsSuccess = true };

            _cli.OnMsgReceviced += stlCli_OnMsgReceviced_StartProgram;
            var resultInt = _cli.Rest();
            _cli.OnMsgReceviced -= stlCli_OnMsgReceviced_StartProgram;

            result.IsSuccess = resultInt == 0;
            return result;

            void stlCli_OnMsgReceviced_StartProgram(string msg)
            {
                if (msg is null)
                    return;

                if (msg.Contains("Unable to connect to ST-LINK!"))
                {
                    result.ErrorMsg = "ERROR: Unable to connect to ST-LINK!";
                }
                OnMsgReceviced?.Invoke(msg);
            }
        }

        public Result Check(string filePath)
        {
            if (!File.Exists(filePath))
            {
                return new Result()
                {
                    IsSuccess = false,
                    ErrorMsg = "ERROR: Check file not found",
                };
            }

            var result = new Result() { IsSuccess = true };

            _cli.OnMsgReceviced += stlCli_OnMsgReceviced_StartProgram;
            var resultInt = _cli.Check(filePath);
            _cli.OnMsgReceviced -= stlCli_OnMsgReceviced_StartProgram;

            result.IsSuccess = resultInt == 0;
            return result;

            void stlCli_OnMsgReceviced_StartProgram(string msg)
            {
                if (msg is null)
                    return;

                if (msg.Contains("Unable to connect to ST-LINK!"))
                {
                    result.ErrorMsg = "ERROR: Unable to connect to ST-LINK!";
                }
                OnMsgReceviced?.Invoke(msg);
            }
        }

        public Result GetCheckSum(string filePath)
        {
            if (!File.Exists(filePath))
            {
                return new Result()
                {
                    IsSuccess = false,
                    ErrorMsg = "ERROR: Check file not found",
                };
            }

            var result = new Result() { IsSuccess = true };

            _cli.OnMsgReceviced += stlCli_OnMsgReceviced_StartProgram;
            var resultInt = _cli.GetCheckSum(filePath);
            _cli.OnMsgReceviced -= stlCli_OnMsgReceviced_StartProgram;

            result.IsSuccess = resultInt == 0;
            return result;

            void stlCli_OnMsgReceviced_StartProgram(string msg)
            {
                if (msg is null)
                    return;

                Regex checksumRegex = new Regex("^(.*)file checksum : (.*)$");
                var match = checksumRegex.Match(msg);
                if (match.Success &&
                    match.Groups.Count > 2 &&
                    match.Groups[2].Value.StartsWith("0x"))
                {
                    var checksum = match.Groups[2].Value;
                    result.Data.Add("checksum", checksum);
                    result.Data.Add("fileName", Path.GetFileName(filePath));

                    OnMsgReceviced?.Invoke($"checksum: {checksum}");
                }

                if (msg.Contains("Unable to connect to ST-LINK!"))
                {
                    result.ErrorMsg = "ERROR: Unable to connect to ST-LINK!";
                }
                OnMsgReceviced?.Invoke(msg);
            }
        }
    }
}
